﻿using System;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.DatabaseServices;
//using Autodesk.AutoCAD.EditorInput;

namespace AcDgLibrary
{
    public class AcDgLineType
    {
        private Document _dwg;
        private Database _database;
        private Transaction _transaction;

        private ObjectId _objectid = ObjectId.Null;

        private AcDgEnums.LineTypes _linetype;

        public AcDgLineType(Document dwg, Transaction trans, AcDgEnums.LineTypes lineType)
        {
            _dwg = dwg;
            _database = _dwg.Database;
            _transaction = trans;

            _linetype = lineType;
        }

        public ObjectId GetLineTypeObjectId
        {
            get
            {
                LinetypeTable lt = (LinetypeTable)_transaction.GetObject(_database.LinetypeTableId, OpenMode.ForWrite);
                if (lt.Has(AcDgEnums.AcDgEnumTranslation.LineTypeEnum(_linetype)))
                {
                    _objectid = lt[AcDgEnums.AcDgEnumTranslation.LineTypeEnum(_linetype)];

                    return _objectid;
                }
                else
                {
                    return LoadLinetypeFromFile();
                }
            }
        }

        private ObjectId LoadLinetypeFromFile()
        {
            ObjectId linetypeid = ObjectId.Null;

            HostApplicationServices.WorkingDatabase.LoadLineTypeFile(AcDgEnums.AcDgEnumTranslation.LineTypeEnum(_linetype), "acad.lin");

            LinetypeTable ltr = (LinetypeTable)_transaction.GetObject(_database.LinetypeTableId, OpenMode.ForRead);
            if (true == ltr.Has(AcDgEnums.AcDgEnumTranslation.LineTypeEnum(_linetype)))
                linetypeid = ltr[AcDgEnums.AcDgEnumTranslation.LineTypeEnum(_linetype)];
            
            return linetypeid;
        }
    }
}
