﻿using System;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.DatabaseServices;

namespace AcDgLibrary
{
    public class AcDgTextStyle
    {
        private Document _dwg;
        private Database _database;
        private Transaction _transaction;

        private string _name = "TextStyle";
        private string _fontname = "ISOCP3";

        private AnnotativeStates _annotative = AnnotativeStates.False;
        private bool _bold = false;
        private bool _isshapefile = false;
        private bool _isvertical = false;
        private bool _italic = false;
        private double _obliquingangle = 0.0;
        private double _priorsize = 0.2;
        private double _textsize = 0.0;
        private double _xscale = 1.0;

        private ObjectId _objectid = ObjectId.Null;

        public AcDgTextStyle(Document dwg, Transaction trans)
        {
            _dwg = dwg;
            _database = _dwg.Database;
            _transaction = trans;
        }

        public string Name
        {
            set { _name = value; }
        }

        public string FontName
        {
            set { _fontname = value; }
        }
        
        public AnnotativeStates Annotative
        {
            set { _annotative = value; }
        }

        public bool Bold
        {
            set { _bold = value; }
        }

        public bool IsShapeFile
        {
            set { _isshapefile = value; }
        }

        public bool IsVertical
        {
            set { _isvertical = value; }
        }

        public bool Italic
        {
            set { _italic = value; }
        }

        public double ObliquingAngle
        {
            set { _obliquingangle = value; }
        }

        public double PriorSize
        {
            set { _priorsize = value; }
        }

        public double TextSize
        {
            set { _textsize = value; }
        }

        public double XScale
        {
            set { _xscale = value; }
        }

        public bool HasTextStyle
        {
            get
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    TextStyleTable tt = (TextStyleTable)_transaction.GetObject(_database.LayerTableId, OpenMode.ForRead);
                    if (tt.Has(_name)) return true;
                    else return false;
                }
            }
        }

        public TextStyleTableRecord GetTextStyleTableRecord
        {
            get
            {
                TextStyleTableRecord ttr = null;

                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    TextStyleTable tt = (TextStyleTable)_transaction.GetObject(_database.TextStyleTableId, OpenMode.ForWrite);
                    if (tt.Has(_name))
                    {
                        ttr = (TextStyleTableRecord)_transaction.GetObject(tt[_name], OpenMode.ForWrite);
                        return ttr;
                    }
                    else
                    {
                        return ttr;
                    }
                }
            }
        }

        public ObjectId Create()
        {
            TextStyleTable tt = (TextStyleTable)_transaction.GetObject(_database.TextStyleTableId, OpenMode.ForWrite);
            if (tt.Has(_name))
            {
                _objectid = tt[_name];
            }
            else
            {
                TextStyleTableRecord ttr = new TextStyleTableRecord();
                ttr.Name = _name;
                ttr.Annotative = _annotative;
                ttr.Font = new Autodesk.AutoCAD.GraphicsInterface.FontDescriptor(_fontname, _bold, _italic, 0, 0);
                ttr.IsShapeFile = _isshapefile;
                ttr.IsVertical = _isvertical;
                ttr.PriorSize = _priorsize;                              
                ttr.ObliquingAngle = _obliquingangle;
                ttr.TextSize = _textsize;
                ttr.XScale = _xscale;                

                _objectid = tt.Add(ttr);
                _transaction.AddNewlyCreatedDBObject(ttr, true);
            }
            return _objectid;
        }
    }
}
