﻿using System;

using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Runtime;

using AutoCAD = Autodesk.AutoCAD.ApplicationServices.Application;

using AcDgLibrary;

[assembly: CommandClass(typeof(AcDgTester.Command))]
namespace AcDgTester
{
    public class Command
    {
        Document _dwg = AutoCAD.DocumentManager.MdiActiveDocument;

        [CommandMethod("CreateNewLayer")]
        public void CreateNewLayer()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgLayer la = new AcDgLayer(_dwg, trans);
                        la.Name = "Layer1";
                        la.Linetype = AcDgEnums.LineTypes.DIVIDEX2;
                        la.Create();

                        trans.Commit();
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("ChangeLayerProperties")]
        public void ChangeLayerProperties()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgLayer la = new AcDgLayer(_dwg, trans);
                        la.Name = "Layer1";
                      
                        LayerTableRecord ltr = la.GetLayerTableRecord;
                        if (ltr != null)
                        {
                            ltr.IsPlottable = false;
                            trans.Commit();
                        }
                        else
                        {
                            return;
                        }
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("CreateNewTextStyle")]
        public void CreateNewTextStyle()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgTextStyle ts = new AcDgTextStyle(_dwg, trans);
                        ts.Name = "TextStyle1";
                        ts.FontName = "Arial";
                        ts.Annotative = AnnotativeStates.True;
                        ts.Bold = true;
                        ts.Italic = true;
                        ts.Create();

                        trans.Commit();
                    }
                }
            }
            catch(SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("ChangeTextStyleProperties")]
        public void ChangeTextStyleProperties()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgTextStyle ts = new AcDgTextStyle(_dwg, trans);
                        ts.Name = "TextStyle1";

                        TextStyleTableRecord ttr = ts.GetTextStyleTableRecord;
                        if (ttr != null)
                        {
                            ttr.Annotative = AnnotativeStates.False;
                            trans.Commit();
                        }
                        else
                        {
                            return;
                        }
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("InsertBlock")]
        public void InsertBlock()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgBlock bl = new AcDgBlock(_dwg, trans);
                        bl.DwgFileName = "c:\\temp\\xrecdwg.dwg";
                        bl.AddBlockToBlockTable();

                        trans.Commit();
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("NewBlockReference")]
        public void NewBlockReference()
        {
            try
            {
                using (DocumentLock dwglck = _dwg.LockDocument())
                {
                    using (Transaction trans = _dwg.TransactionManager.StartTransaction())
                    {
                        AcDgBlock bl = new AcDgBlock(_dwg, trans);
                        bl.BlockName = "xrecdwg";
                        bl.InsertionPoint = new Point3d(0, 0, 0);
                        bl.CreateNewBlockReference();

                        trans.Commit();
                    }
                }
            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }

        [CommandMethod("DwgThumbnailPreview")]
        public void DwgThumbnailPreview()
        {
            try
            {
                Form1 dlg = new Form1();
                dlg.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
                AutoCAD.ShowModalDialog(dlg);

            }
            catch (SystemException ex)
            {
                AutoCAD.ShowAlertDialog(ex.Message);
            }
        }
    }
}
